from __future__ import print_function
import logging
import warnings

from pyramid.view import view_config

from . import def_name

from schema.model import ScienceProductComment, ProblemSeverityList, ProblemType
from services import DB
from sqlalchemy import exc as sa_exc, asc, desc

log = logging.getLogger(__name__)


@view_config(route_name='data_quality_notes_view', renderer='json')
def data_quality_notes_view(request):
    p = request.params
    log.info(f'{p}')
    notes = get_data_quality_notes(**request.params)
    # notes = get_data_quality_notes_from_spl_list(**request.params)
    return notes


def get_data_quality_notes(**params):
    try:
        with warnings.catch_warnings():
            warnings.simplefilter("ignore", category=sa_exc.SAWarning)
            spl = params.get('spl')
            notes = DB['SDM'].query(ScienceProductComment, ProblemType, ProblemSeverityList) \
                .join(ProblemSeverityList, ScienceProductComment.severity == ProblemSeverityList.severity) \
                .join(ProblemType) \
                .filter(ScienceProductComment.science_product_locator == spl) \
                .order_by(desc(ScienceProductComment.severity)) \
                .all()
            notes_list = [dict(comment=r.ScienceProductComment.comment,
                               severity=r.ScienceProductComment.severity,
                               severity_desc=r.ProblemSeverityList.description,
                               problem=r.ProblemType.description) for r in notes]
            log.debug(f'getting notes, but schema is not updated, for: {spl}')
            return notes_list
    except Exception as x:
        ex = 'Exception caught in {0}: {1}'.format(def_name(), str(x))
        raise Exception(ex)


def get_data_quality_notes_from_spl_list(**params):
    try:
        with warnings.catch_warnings():
            warnings.simplefilter("ignore", category=sa_exc.SAWarning)
            spl = params.get('spl').split(',')
            notes = DB['SDM'].query(ScienceProductComment, ProblemType, ProblemSeverityList) \
                .join(ProblemSeverityList, ScienceProductComment.severity == ProblemSeverityList.severity) \
                .join(ProblemType) \
                .filter(ScienceProductComment.science_product_locator.in_(spl)) \
                .order_by(asc(ScienceProductComment.science_product_locator), desc(ScienceProductComment.severity)) \
                .all()

            notes_dict = dict()
            for res in notes:
                if notes_dict.get(res.ScienceProductComment.science_product_locator, None) is None:
                    notes_dict[res.ScienceProductComment.science_product_locator] = [
                        dict(comment=res.ScienceProductComment.comment,
                             severity=res.ScienceProductComment.severity,
                             severity_desc=res.ProblemSeverityList.description,
                             problem=res.ProblemType.description)]
                else:
                    notes_dict[res.ScienceProductComment.science_product_locator].append(dict(
                        comment=res.ScienceProductComment.comment,
                        severity=res.ScienceProductComment.severity,
                        severity_desc=res.ProblemSeverityList.description,
                        problem=res.ProblemType.description))
        return notes_dict

    except Exception as x:
        ex = 'Exception caught in {0}: {1}'.format(def_name(), str(x))
        raise Exception(ex)
